@@ -1,5 +1,6 @@
 local luacheck = require "luacheck"
 local argparse = require "luacheck.argparse"
+local builtin_standards = require "luacheck.builtin_standards"
 local config = require "luacheck.config"
 local options = require "luacheck.options"
 local expand_rockspec = require "luacheck.expand_rockspec"
@@ -16,15 +17,6 @@ local function critical(msg)
    os.exit(3)
 end
 
-local function global_error_handler(err)
-   if type(err) == "table" and err.pattern then
-      critical("Invalid pattern '" .. err.pattern .. "'")
-   else
-      critical(debug.traceback(
-         ("Luacheck %s bug (please report at github.com/mpeterv/luacheck/issues):\n%s"):format(luacheck._VERSION, err), 2))
-   end
-end
-
 local function main()
    local default_cache_path = ".luacheckcache"
 
@@ -53,12 +45,22 @@ Equivalent to --ignore 4.]]):target("redefined"):action("store_false")
 loop variables. Equivalent to --ignore 21[23].]]):target("unused_args"):action("store_false")
       parser:flag("-s --no-unused-secondaries", [[Filter out warnings related to unused variables set
 together with used ones.]]):target("unused_secondaries"):action("store_false")
-      parser:flag("--no-self", "Filter out warnings related to implicit self argument."):target("self"):action("store_false")
+      parser:flag("--no-self", "Filter out warnings related to implicit self argument.")
+         :target("self"):action("store_false")
 
-      parser:option("--std", [[Set standard globals. <std> can be one of:
-   _G (default) - globals of the current Lua
-      interpreter;
-   lua51 - globals of Lua 5.1;
+      local default_std_name = "max"
+
+      for _, name in ipairs({"lua51c", "lua52c", "lua53c", "luajit"}) do
+         if builtin_standards._G == builtin_standards[name] then
+            default_std_name = name
+            break
+         end
+      end
+
+      parser:option("--std", ([[Set standard globals, default is _G.
+<std> can be one of:
+   lua51 - globals of Lua 5.1 without deprecated ones;
+   lua51c - globals of Lua 5.1;
    lua52 - globals of Lua 5.2;
    lua52c - globals of Lua 5.2 with LUA_COMPAT_ALL;
    lua53 - globals of Lua 5.3;
@@ -66,39 +68,50 @@ together with used ones.]]):target("unused_secondaries"):action("store_false")
    luajit - globals of LuaJIT 2.0;
    ngx_lua - globals of Openresty lua-nginx-module
       with LuaJIT 2.0;
-   rockspec - globals allowed in rockspecs;
    min - intersection of globals of Lua 5.1, Lua 5.2,
       Lua 5.3 and LuaJIT 2.0;
    max - union of globals of Lua 5.1, Lua 5.2, Lua 5.3
       and LuaJIT 2.0;
+   _G - same as lua51c, lua52c, lua53c, or luajit
+      depending on version of Lua used to run luacheck
+      or same as max if couldn't detect the version.
+      Currently %s;
    busted - globals added by Busted 2.0;
+   rockspec - globals allowed in rockspecs;
    none - no standard globals.
 
-   Sets can be combined using "+".]])
-      parser:option("--globals", "Add custom globals on top of standard ones.")
+   Sets can be combined using "+".]]):format(default_std_name))
+      parser:option("--globals", [[Add custom global variables (e.g. foo) or
+fields (e.g. foo.bar) on top of standard ones.]])
          :args "*"
          :count "*"
-         :argname "<global>"
+         :argname "<name>"
          :action "concat"
          :init(nil)
-      parser:option("--read-globals", "Add read-only globals.")
+      parser:option("--read-globals", "Add read-only global variables or fields.")
          :args "*"
          :count "*"
-         :argname "<global>"
+         :argname "<name>"
          :action "concat"
          :init(nil)
-      parser:option("--new-globals", [[Set custom globals. Removes custom globals added
-previously.]])
+      parser:option("--new-globals", [[Set custom global variables or fields. Removes
+custom globals added previously.]])
          :args "*"
          :count "*"
-         :argname "<global>"
+         :argname "<name>"
          :action "concat"
          :init(nil)
-      parser:option("--new-read-globals", [[Set read-only globals. Removes read-only globals added
-previously.]])
+      parser:option("--new-read-globals", [[Set read-only global variables or fields. Removes
+read-only globals added previously.]])
          :args "*"
          :count "*"
-         :argname "<global>"
+         :argname "<name>"
+         :action "concat"
+         :init(nil)
+      parser:option("--not-globals", "Remove custom and standard global variables or fields.")
+         :args "*"
+         :count "*"
+         :argname "<name>"
          :action "concat"
          :init(nil)
       parser:flag("-c --compat", "Equivalent to --std max.")
@@ -108,6 +121,13 @@ the top level scope.]])
       parser:flag("-m --module", [[Limit visibility of implicitly defined globals to
 their files.]])
 
+      parser:option("--max-line-length", "Set maximum allowed line length (default: 120).")
+         :argname "<length>"
+         :convert(tonumber)
+      parser:flag("--no-max-line-length", "Do not limit line length.")
+         :action "store_false"
+         :target "max_line_length"
+
       parser:option("--ignore -i", [[Filter out warnings matching these patterns.
 If a pattern contains slash, part before slash matches
 warning code and part after it matches name of related
@@ -204,7 +224,8 @@ patterns.]])
    end
 
    local function is_included(args, name)
-      return not match_any(args.exclude_files, name) and (#args.include_files == 0 or match_any(args.include_files, name))
+      return not match_any(args.exclude_files, name) and (
+         #args.include_files == 0 or match_any(args.include_files, name))
    end
 
    -- Expands folders, rockspecs, -
@@ -340,7 +361,8 @@ patterns.]])
          ("Couldn't load cache from %s: data corrupted"):format(cache_filename))
    end
 
-   -- Returns sparse array of sources of files that need to be checked, updates bad_files with files that had I/O issues.
+   -- Returns sparse array of sources of files that need to be checked,
+   -- updates bad_files with files that had I/O issues.
    local function get_srcs_to_check(cached_reports, files, bad_files)
       local res = {}
 
@@ -491,7 +513,12 @@ patterns.]])
    end
 
    local parser = get_parser()
-   local args = parser:parse()
+   local ok, args = parser:pparse()
+   if not ok then
+      io.stderr:write(("%s\n\nError: %s\n"):format(parser:get_usage(), args))
+      os.exit(3)
+   end
+
    local conf
 
    if args.no_config then
@@ -536,4 +563,16 @@ patterns.]])
    os.exit(exit_code)
 end
 
-xpcall(main, global_error_handler)
+local _, error_wrapper = utils.try(main)
+local err = error_wrapper.err
+local traceback = error_wrapper.traceback
+
+if utils.is_instance(err, utils.InvalidPatternError) then
+   critical(("Invalid pattern '%s'"):format(err.pattern))
+elseif type(err) == "string" and err:match("interrupted!$") then
+   critical("Interrupted")
+else
+   local msg = ("Luacheck %s bug (please report at github.com/mpeterv/luacheck/issues):\n%s\n%s"):format(
+      luacheck._VERSION, err, traceback)
+   critical(msg)
+end
